﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Drawing;
using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Common.Resources;
using HeuristicLab.Core;
using HeuristicLab.Encodings.BinaryVectorEncoding;
using HeuristicLab.Encodings.RealVectorEncoding;

namespace HeuristicLab.Analysis.FitnessLandscape.DistanceCalculators {

  [Item("RealVectorDistanceCalculator", "Calculates the Euclidian distance of two real vectors")]
  [StorableType("26271DB0-0868-4363-84AE-46CF583276F0")]
  public class RealVectorDistanceCalculator : NamedItem, IItemDistanceCalculator {

    #region Properties

    public override bool CanChangeName { get { return false; } }
    public override bool CanChangeDescription { get { return false; } }
    public static new Image StaticItemImage { get { return VSImageLibrary.Function; } }

    #endregion

    #region Construction & Cloning

    [StorableConstructor]
    protected RealVectorDistanceCalculator(StorableConstructorFlag _) : base(_) { }

    protected RealVectorDistanceCalculator(RealVectorDistanceCalculator original, Cloner cloner)
      : base(original, cloner) {
    }

    public RealVectorDistanceCalculator() {
      name = ItemName;
      description = ItemDescription;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new RealVectorDistanceCalculator(this, cloner);
    }

    #endregion

    #region IItemDistanceCalculator Members

    public Type ItemType {
      get { return typeof(BinaryVector); }
    }

    public double Distance(IItem x, IItem y) {
      RealVector a = (RealVector)x;
      RealVector b = (RealVector)y;
      if (a.Length != b.Length)
        throw new InvalidOperationException("Cannot compare vectors of different lengths");
      double sqrSum = 0;
      for (int i = 0; i < a.Length; i++) {
        sqrSum = Sqr(a[i] - b[i]);
      }
      return Math.Sqrt(sqrSum);
    }

    private static double Sqr(double x) { return x * x; }

    #endregion
  }
}
